/*
 * Based on powermate-1.0 by William R. Sowerbutts
 * Leigh L Klotz, Jr. <klotz@graflex.org>
 * Instructions:
 *  . Turn knob to set brightness or pulse rate.
 *  . Tap to change between brightness and pulse rate.
 * Problems:
 *  . Seems to hang when pulse gets up to about 300.  Just ^C out.
 *  . Limit logic on parameters is dup'd in powermate_pulse_led and
 *    process_event.  Eventually a library should have a clean interface
 *    and simple programs like this can use the library.
 */

#include <linux/input.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include "findpowermate.h"

void update();

int flag = 0;
int abs_offset = 0;
int powermate = -1;
int brightness = 128;
int pulse_speed=255;
int pulse_table=0; 
int pulse_asleep=0;
int pulse_awake=0;

void process_event(struct input_event *ev)
{
  int updatep = 1;

#ifdef VERBOSE
  fprintf(stderr, "type=0x%04x, code=0x%04x, value=%d\n",
	  ev->type, ev->code, (int)ev->value);
#endif

  switch(ev->type){
  case EV_SYN:
#ifdef VERBOSE
    printf("EV_SYN marker; code=0x%04x, value=0x%08x\n", ev->code, ev->value);
#endif
    updatep = 0;
    break;
  case EV_MSC:
#ifdef VERBOSE
    printf("The LED pulse settings were changed; code=0x%04x, value=0x%08x\n", ev->code, ev->value);
#endif
    updatep = 0;
    break;
  case EV_REL:
    if(ev->code != REL_DIAL) {
#ifdef VERBOSE
      fprintf(stderr, "Warning: unexpected rotation event; ev->code = 0x%04x\n", ev->code);
#endif
    } else {
      abs_offset += (int)ev->value;
#ifdef VERBOSE
      printf("Button was rotated %d units; Offset from start is now %d units\n", (int)ev->value, abs_offset);
#endif
      if (flag == 0) {
	brightness += (char)ev->value;
	if (brightness < 0) brightness = 0;
	if (brightness > 255) brightness = 255;
	fprintf(stderr, "Brightness %d\n", brightness);
      } else {
	pulse_speed += (int)ev->value;
	if (pulse_speed < 0) pulse_speed = 0;
	if (pulse_speed > 280) pulse_speed = 280;
	fprintf(stderr, "pulse_speed %d\n", pulse_speed);
      }
    }
    break;
  case EV_KEY:
    if(ev->code != BTN_0) {
#ifdef VERBOSE
      fprintf(stderr, "Warning: unexpected key event; ev->code = 0x%04x\n", ev->code);
#endif
    }
    else {
#ifdef VERBOSE
      printf("Button was %s\n", ev->value? "pressed":"released");
#endif
      if (ev->value) flag = !flag;
#ifdef VERBOSE
      printf("Flag=%d\n", flag);
#endif

    }
    break;
  default:
    fprintf(stderr, "Warning: unexpected event type; ev->type = 0x%04x\n", ev->type);
  }
  if (updatep) update();

  fflush(stdout);
}
void update() {
  if (flag) pulse_awake = 1; else pulse_awake=0;
  powermate_pulse_led(powermate, brightness, pulse_speed, pulse_table, pulse_asleep, pulse_awake);
}


#define BUFFER_SIZE 32
void watch_powermate(int fd)
{
  struct input_event ibuffer[BUFFER_SIZE];
  int r, events, i;

  update();

  while(1){
    r = read(fd, ibuffer, sizeof(struct input_event) * BUFFER_SIZE);
    if( r > 0 ){
      events = r / sizeof(struct input_event);
      for(i=0; i<events; i++)
	process_event(&ibuffer[i]);
    }else{
      fprintf(stderr, "read() failed: %s\n", strerror(errno));
      return;
    }
  }
}

int main(int argc, char *argv[])
{
  if(argc == 1)
    powermate = find_powermate(O_RDWR);
  else
    powermate = open_powermate(argv[1], O_RDWR);

  if(powermate < 0){
    fprintf(stderr, "Unable to locate powermate\n");
    fprintf(stderr, "Try: %s [device]\n", argv[0]);
    return 1;
  }

  watch_powermate(powermate);

  close(powermate);

  return 0;
}
